//---------------------------------
// 2018/6/3

#include "WebPageHandler.h"
#include "TSL2561I2C.h"
#include "StrList.h"

#ifdef STORAGE_SPIFFS
#include <FS.h>
#else
#include <SPI.h>
#include "SdFat.h"
#endif

//---------------------------------
// Constant

const char WebPageHandler::luxLogFileName[] = "lux.csv";
const char WebPageHandler::pageCamera[] = "camera.php";
const char WebPageHandler::pageCameraSetting[] = "cameraSetting.php";
const char WebPageHandler::pageTSL2561[] = "tsl2561.php";
const char WebPageHandler::pageMotionSensor[] = "motionSensor.php";
const char WebPageHandler::pageSystem[] = "system.php";


//---------------------------------
// Private Class Function

String WebPageHandler::getContentType(String filename){
  if(server->hasArg("download")) return "application/octet-stream";
  else if(filename.endsWith(".htm")) return "text/html";
  else if(filename.endsWith(".html")) return "text/html";
  else if(filename.endsWith(".css")) return "text/css";
  else if(filename.endsWith(".js")) return "application/javascript";
  else if(filename.endsWith(".png")) return "image/png";
  else if(filename.endsWith(".gif")) return "image/gif";
  else if(filename.endsWith(".jpg")) return "image/jpeg";
  else if(filename.endsWith(".ico")) return "image/x-icon";
  else if(filename.endsWith(".xml")) return "text/xml";
  else if(filename.endsWith(".pdf")) return "application/x-pdf";
  else if(filename.endsWith(".zip")) return "application/x-zip";
  else if(filename.endsWith(".gz")) return "application/x-gzip";
  //return "text/plain";
  return "application/octet-stream";
}


// 共通のHTMLヘッダーを送信
void WebPageHandler::sendHeader(){
  server->sendContent(
    "<head>\n"
    "  <meta charset=\"utf-8\">\n"
    "  <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\n"
    "  <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\n"
    "  <title>ESP-SensorCam</title>\n\n"
    
    "  <!-- Bootswatch flatly -->\n"
    "  <link href=\"https://maxcdn.bootstrapcdn.com/bootswatch/3.3.7/flatly/bootstrap.min.css\" rel=\"stylesheet\" integrity=\"sha384-+ENW/yibaokMnme+vBLnHMphUYxHs34h9lpdbSLuAwGkOKFRl4C34WkjazBtb7eT\" crossorigin=\"anonymous\">\n"
    "</head>\n"
  );
}


void WebPageHandler::sendHeaderRefresh(uint8_t t, String link){
  server->sendContent(
    "<head>\n"
    "  <meta charset=\"utf-8\">\n"
    "  <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\n"
    "  <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\n"
    "  <title>ESP-SensorCam</title>\n\n"
    
    "  <!-- Bootswatch flatly -->\n"
    "  <link href=\"https://maxcdn.bootstrapcdn.com/bootswatch/3.3.7/flatly/bootstrap.min.css\" rel=\"stylesheet\" integrity=\"sha384-+ENW/yibaokMnme+vBLnHMphUYxHs34h9lpdbSLuAwGkOKFRl4C34WkjazBtb7eT\" crossorigin=\"anonymous\">\n"
  );
  server->sendContent(String("  <meta http-equiv=\"refresh\" content=\"") + 
    String(t) + ";URL=" + link + "\">\n" + 
    "</head>\n"
  );
}


// 共通のナビゲーションバーを送信
void WebPageHandler::sendNavBar(const char*page){
  server->sendContent(
    "<nav class=\"navbar navbar-default navbar-fixed-top\">\n"
    "  <div class=\"container-fluid\">\n"
    "    <div class=\"navbar-header\">\n"
    "      <button type=\"button\" class=\"navbar-toggle collapsed\" data-toggle=\"collapse\" data-target=\"#bs-example-navbar-collapse-1\">\n"
    "        <span class=\"sr-only\">Toggle navigation</span>\n"
    "        <span class=\"icon-bar\"></span>\n"
    "        <span class=\"icon-bar\"></span>\n"
    "        <span class=\"icon-bar\"></span>\n"
    "      </button>\n"
    "      <a class=\"navbar-brand\" href=\"#\">ESP-SensorCam</a>\n"
    "    </div>\n\n"
  );
  server->sendContent(
    "    <div class=\"collapse navbar-collapse\" id=\"bs-example-navbar-collapse-1\">\n"
    "      <ul class=\"nav navbar-nav\">\n"
  );
  
  if(0==strcmp(page, pageCamera)){
    server->sendContent("        <li class=\"active\"><a href=\"" + String(pageCamera) + "\">カメラ<span class=\"sr-only\">(current)</span></a></li>\n");
  }else{
    server->sendContent("        <li><a href=\"" + String(pageCamera) + "\">カメラ</a></li>\n");
  }
  if(0==strcmp(page, pageCameraSetting)){
    server->sendContent("        <li class=\"active\"><a href=\"" + String(pageCameraSetting) + "\">カメラ設定<span class=\"sr-only\">(current)</span></a></li>\n");
  }else{
    server->sendContent("        <li><a href=\"" + String(pageCameraSetting) + "\">カメラ設定</a></li>\n");
  }
  if(0==strcmp(page, pageTSL2561)){
    server->sendContent("        <li class=\"active\"><a href=\"" + String(pageTSL2561) + "\">TSL2561<span class=\"sr-only\">(current)</span></a></li>\n");
  }else{
    server->sendContent("        <li><a href=\"" + String(pageTSL2561) + "\">TSL2561</a></li>\n");
  }
  if(0==strcmp(page, pageMotionSensor)){
    server->sendContent("        <li class=\"active\"><a href=\"" + String(pageMotionSensor) + "\">モーションセンサ<span class=\"sr-only\">(current)</span></a></li>\n");
  }else{
    server->sendContent("        <li><a href=\"" + String(pageMotionSensor) + "\">モーションセンサ</a></li>\n");
  }
  if(0==strcmp(page, pageSystem)){
    server->sendContent("        <li class=\"active\"><a href=\"" + String(pageSystem) + "\">システム<span class=\"sr-only\">(current)</span></a></li>\n");
  }else{
    server->sendContent("        <li><a href=\"" + String(pageSystem) + "\">システム</a></li>\n");
  }
  server->sendContent(
    "      </ul>\n"
    "    </div>\n"
    "  </div>\n"
    "</nav>\n\n"
  );
}


// スクリプト情報を送信
void WebPageHandler::sendScript(){
  server->sendContent(
    "<script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js\"></script>\n"
    "<script src=\"https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js\" integrity=\"sha384-Tc5IQib027qvyjSMfHjOMaLkfuWVxZxUPnCJA7l2mCWNIpG9mGCD8wGNIcPD7Txa\" crossorigin=\"anonymous\"></script>\n"
  );
}


// カメラページを送信
void WebPageHandler::handleCameraPage(){
  #ifdef STORAGE_SPIFFS
  #else
  SdFat*SD = new SdFat;
  #endif
  
  char fname[maxFileName];
  String showPhoto = "";
  String filelistStr = server->arg("filelist");
  String actionStr = server->arg("action");
  
  if(actionStr == "takephoto"){
    // Skip if lux is very low
    TSL2561I2C tsl2561(pinSDA, pinSCL);
    bool tslStat = tsl2561.meas();
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Take Photo");
    showPhoto = timelyTaskManager->getTimeFileName(TimelyTaskManager::formatSecond)+".jpg";
    if(arduCamManager->takePhoto(showPhoto)){
      Storage::opLog(" Success\n");
    }else{
      showPhoto = "";
      Storage::opLog(" Failed\n");
    }
  }else if(actionStr == "select"){
    showPhoto = filelistStr;
  }else if(actionStr == "delete"){
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Delete Photo " + filelistStr + "\n");
    String path = arduCamManager->storeDir + filelistStr;
    #ifdef STORAGE_SPIFFS
    if(SPIFFS.begin()){
      if(SPIFFS.exists(path)){
        SPIFFS.remove(path);
      }
      SPIFFS.end();
    }
    #else
    if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
      if(SD->exists(path.c_str())){
        SD->remove(path.c_str());
      }
    }
    #endif
  }else if(actionStr == "deleteall"){
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Delete All Photo\n");
    #ifdef STORAGE_SPIFFS
    if(SPIFFS.begin()){
      Dir dir = SPIFFS.openDir(arduCamManager->storeDir);
      while(dir.next()){
        SPIFFS.remove(dir.fileName());
      }
      SPIFFS.end();
    }
    #else
    if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
      File dir = SD->open(arduCamManager->storeDir);
      if(dir){
        if(dir.isDirectory()){
          while(1){
            delay(0);
            File file = dir.openNextFile();
            if(!file){
              break;
            }
            file.getName(fname, maxFileName);
            if(file.isFile()){
              file.close();
              SD->remove((arduCamManager->storeDir + fname).c_str());
            }else{
              file.close();
            }
          }
        }
        dir.close();
      }
    }
    #endif
  }

  server->setContentLength(CONTENT_LENGTH_UNKNOWN);
  server->send(200, "text/html", "");
  
  server->sendContent(
    "<!DOCTYPE html>\n\n"
    "<html lang=\"ja\">\n\n"
  );
  sendHeader();
  server->sendContent(
    "<body style=\"padding-top:70px\">\n\n"
  );
  sendNavBar(pageCamera);
  
  server->sendContent(
    "<div class=\"container\">\n"
    "  <div class=\"row\"><div class=\"col-sm-10 col-md-8\">\n"
  );
  
  server->sendContent(
    "    <form method=\"POST\" class=\"form-horizontal\">\n"
    "      <div class=\"form-group\"><div class=\"col-xs-12\">\n"
    "        <select class=\"form-control\" size=\"8\" name=\"filelist\">\n"
  );
  
  // Read photo directory
  uint16_t photoCount = 0;
  StrList*strList = new StrList();

  #ifdef STORAGE_SPIFFS
  if(SPIFFS.begin()){
    Dir dir = SPIFFS.openDir(arduCamManager->storeDir);
    String fileName;
    while(dir.next()){
      fileName = dir.fileName();
      fileName = fileName.substring(fileName.lastIndexOf("/")+1);
      strList->add(fileName);
      photoCount++;
    }
    SPIFFS.end();
  }
  #else
  if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
    File dir = SD->open(arduCamManager->storeDir);
    if(dir){
      if(dir.isDirectory()){
        while(1){
          delay(0);
          File file = dir.openNextFile();
          if(!file){
            break;
          }
          file.getName(fname, maxFileName);
          if(file.isFile()){
            strList->add(String(fname));
            photoCount++;
          }
          file.close();
        }
      }
      dir.close();
    }
  }
  delete SD;
  #endif

  if(showPhoto.length()==0 && 0<photoCount){
    showPhoto = strList->getLast()->str;
  }

  StrListItem*strListItem = strList->getFirst();

  for(uint16_t i=0; i<photoCount; i++){
    if((strListItem->str == showPhoto)){
      server->sendContent("          <option selected=\"selected\">" + strListItem->str + "</option>\n");
    }else{
      server->sendContent("          <option>" + strListItem->str + "</option>\n");
    }
    strListItem = strList->getNext();
  }
  delete strList;
  
  server->sendContent(
    "        </select>\n"
    "      </div></div>\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"select\" class=\"btn btn-default btn-block\">選択</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"delete\" class=\"btn btn-default btn-block\">削除</button>\n"
    "        </div>\n"
    "      </div>\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" data-toggle=\"modal\" data-target=\"#deleteAllDialog\">全て削除</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"takephoto\" class=\"btn btn-default btn-block\">撮影</button>\n"
    "        </div>\n"
    "      </div>\n"
    "    </form>\n"
  );

  if(photoCount > 0){
    server->sendContent(
      //"    <div class=\"well\">\n"
      "      <img src=\"" + arduCamManager->storeDir + showPhoto + "\" class=\"img-responsive\">\n"
      //"    </div>\n"
    );
  }else{
    server->sendContent(
      "    <h3>Photo</h3>\n"
      "    <div class=\"well\">\n"
      "      <p>写真がありません</p>\n"
      "    </div>\n"
    );
  }
  
  server->sendContent(
    "  </div></div><!-- row, col -->\n"
    "</div>\n"
  );

  server->sendContent(
    "<div class=\"modal\" id=\"deleteAllDialog\">\n"
    "  <div class=\"modal-dialog\">\n"
    "    <div class=\"modal-content\">\n"
    "      <div class=\"modal-header\">\n"
    "        <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\">&times;</button>\n"
    "        <h4 class=\"modal-title\">全て削除しますか？</h4>\n"
    "      </div>\n"
    "      <!-- <div class=\"modal-body\">\n"
    "      </div> -->\n"
    "      <div class=\"modal-footer\">\n"
    "        <form method=\"POST\">\n"
    "          <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\">キャンセル</button>\n"
    "          <button type=\"submit\" class=\"btn btn-primary\" name=\"action\" value=\"deleteall\">OK</button>\n"
    "        </form>\n"
    "      </div>\n"
    "    </div>\n"
    "  </div>\n"
    "</div>\n\n"
  );

  sendScript();

  server->sendContent(
    "</body>\n"
    "</html>\n"
  );
}


// カメラ設定ページを送信
void WebPageHandler::handleCameraSettingPage(){
  String sizeStr = server->arg("size");
  String qualityStr = server->arg("quality");

  if(server->arg("action") == "save"){
    uint8_t jpegSize;
    uint8_t jpegQuality;
    if(qualityStr=="low"){
      jpegQuality = ArduCamManager::qualityLow;
    }
    if(qualityStr=="fair"){
      jpegQuality = ArduCamManager::qualityFair;
    }
    if(qualityStr=="good"){
      jpegQuality = ArduCamManager::qualityGood;
    }

    if(sizeStr=="160x120"){
      jpegSize = 0;
    }
    if(sizeStr=="176x144"){
      jpegSize = 1;
    }
    if(sizeStr=="320x240"){
      jpegSize = 2;
    }
    if(sizeStr=="352x288"){
      jpegSize = 3;
    }
    if(sizeStr=="640x480"){
      jpegSize = 4;
    }
    if(sizeStr=="800x600"){
      jpegSize = 5;
    }
    if(sizeStr=="1024x768"){
      jpegSize = 6;
    }
    if(sizeStr=="1280x1024"){
      jpegSize = 7;
    }
    if(sizeStr=="1600x1200"){
      jpegSize = 8;
    }
    
    arduCamManager->setJpegParam(jpegSize, jpegQuality);
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Save JPEG Setting\n");
  }
  
  server->setContentLength(CONTENT_LENGTH_UNKNOWN);
  server->send(200, "text/html", "");
  
  server->sendContent(
    "<!DOCTYPE html>\n\n"
    "<html lang=\"ja\">\n\n"
  );
  sendHeader();
  server->sendContent(
    "<body style=\"padding-top:70px\">\n\n"
  );
  sendNavBar(pageCameraSetting);
  
  server->sendContent(
    "<div class=\"container\">\n"
    "  <div class=\"row\"><div class=\"col-sm-10 col-md-8\"><div class=\"well\">\n"
  );

  // Form, left column
  server->sendContent(
    "   <form method=\"POST\" class=\"form-horizontal\">\n"
    "      <div class=\"col-xs-12 col-sm-6\">\n"
  );

  // Jpeg size
  server->sendContent(
    "        <div class=\"form-group\">\n"
    "          <label class=\"control-label col-xs-4\">JPEGサイズ</label>\n"
    "          <div class=\"col-xs-8\">\n"
    "            <select class=\"form-control\" name=\"size\" id=\"size\">\n"
  );
  server->sendContent("              <option");
  if(0==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">160x120</option>\n");
  
  server->sendContent("              <option");
  if(1==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">176x144</option>\n");
  
  server->sendContent("              <option");
  if(2==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">320x240</option>\n");
  
  server->sendContent("              <option");
  if(3==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">352x288</option>\n");
  
  server->sendContent("              <option");
  if(4==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">640x480</option>\n");
  
  server->sendContent("              <option");
  if(5==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">800x600</option>\n");
  
  server->sendContent("              <option");
  if(6==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">1024x768</option>\n");
  
  server->sendContent("              <option");
  if(7==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">1280x1024</option>\n");
  
  server->sendContent("              <option");
  if(8==arduCamManager->getJpegSize()){
    server->sendContent(" selected=\"selected\"");
  }
  server->sendContent(">1600x1200</option>\n");
  
  server->sendContent(
    "            </select>\n"
    "          </div>\n"
    "        </div>\n"
    "      </div>\n"
  );


  // Right column
  server->sendContent(
    "      <div class=\"col-xs-12 col-sm-6\">\n"
  );

  // Jpeg quality
  server->sendContent(
    "        <div class=\"form-group\">\n"
    "          <label class=\"control-label col-xs-4\">JPEG品質</label>\n"
    "          <div class=\"col-xs-8\">\n"
    "            <div class=\"radio\">\n"
    "              <label><input type=\"radio\" name=\"quality\" id=\"low\" value=\"low\""
  );
  if(ArduCamManager::qualityLow==arduCamManager->getJpegQuality()){
    server->sendContent(" checked=\"checked\"");
  }
  server->sendContent(">低い</label>\n"
    "            </div>\n"
    "            <div class=\"radio\">\n"
    "              <label><input type=\"radio\" name=\"quality\" id=\"fair\" value=\"fair\""
  );
  if(ArduCamManager::qualityFair==arduCamManager->getJpegQuality()){
    server->sendContent(" checked=\"checked\"");
  }
  server->sendContent(">普通</label>\n"
    "            </div>\n"
    "            <div class=\"radio\">\n"
    "              <label><input type=\"radio\" name=\"quality\" id=\"good\" value=\"good\""
  );
  if(ArduCamManager::qualityGood==arduCamManager->getJpegQuality()){
    server->sendContent(" checked=\"checked\"");
  }
  server->sendContent(">良い</label>\n"
    "            </div>\n"
    "          </div>\n"
    "        </div>\n\n"
  );


  // Close right column
  server->sendContent(
    "      </div>\n\n"
  );

  // Submit and close form
  server->sendContent(
    "      <button type=\"submit\" name=\"action\" value=\"save\" class=\"btn btn-default btn-block\">保存</button>\n"
    "    </form>\n\n"
    "  </div></div></div><!-- row, col -->\n"
    "</div>\n\n"
  );

  sendScript();

  server->sendContent(
    "</body>\n"
    "</html>\n"
  );
}


// モーションセンサーページを送信
void WebPageHandler::handleMotionSensorPage(){
  String actionStr = server->arg("action");
  if(actionStr == "start"){
    if(!motionSensorStatus){
      motionSensorStatus = true;
    }
  }else if(actionStr == "stop"){
    if(motionSensorStatus){
      motionSensorStatus = false;
    }
  }
  
  server->setContentLength(CONTENT_LENGTH_UNKNOWN);
  server->send(200, "text/html", "");
  
  server->sendContent(
    "<!DOCTYPE html>\n\n"
    "<html lang=\"ja\">\n\n"
  );
  sendHeader();
  server->sendContent(
    "<body style=\"padding-top:70px\">\n\n"
  );
  sendNavBar(pageMotionSensor);
  
  server->sendContent(
    "<div class=\"container\">\n"
    "  <div class=\"row\"><div class=\"col-sm-10 col-md-8\">\n"
    "    <div class=\"panel panel-primary\">\n"
    "      <div class=\"panel-heading\">\n"
    "        <h3 class=\"panel-title\">モーションセンサー</h3>\n"
    "      </div>\n"
    "      <div class=\"panel-body\">\n"
  );
  if(motionSensorStatus){
    server->sendContent("        <h4>センサー感知撮影 : 動作中</h4>\n");
  }else{
    server->sendContent("        <h4>センサー感知撮影 : 停止</h4>\n");
  }
  server->sendContent(
    "      </div>\n"
    "    </div>\n"
  );
  server->sendContent(
    "    <form methos=\"POST\" class=\"form-horizontal\">\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"start\" class=\"btn btn-default btn-block\">開始</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"stop\" class=\"btn btn-default btn-block\">停止</button>\n"
    "        </div>\n"
    "      </div>\n"
    "    </form>\n"
    "    <p>本機能は室内または日陰での使用を推奨致します。太陽光など強い光が入ると誤検知することがあります。</p>\n"
  );
  server->sendContent(  
    "  </div></div><!-- row, col -->\n"
    "</div>\n\n"
  );

  sendScript();

  server->sendContent_P(
    "</body>\n"
    "</html>\n"
  );
}


// 照度センサーページを送信
void WebPageHandler::handleTSL2561Page(){
  TSL2561I2C tsl2561(pinSDA, pinSCL);

  String actionStr = server->arg("action");
  if(actionStr == "start"){
    if(!luxLogStatus){
      luxLogStatus = true;
      callBack(luxLogStart);
    }
  }else if(actionStr == "stop"){
    if(luxLogStatus){
      luxLogStatus = false;;
      callBack(luxLogStop);
    }
  }else if(actionStr == "clear"){
    String path = String("/") + luxLogFileName;
    
    #ifdef STORAGE_SPIFFS
    if(SPIFFS.begin()){
      SPIFFS.remove(path);
      SPIFFS.end();
    }
    #else
    SdFat SD;
    if(SD.begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
      if(SD.exists(path.c_str())){
        SD.remove(path.c_str());
      }
    }
    #endif

    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Clear Lux Log\n");
  }
  
  server->setContentLength(CONTENT_LENGTH_UNKNOWN);
  server->send(200, "text/html", "");
  
  server->sendContent(
    "<!DOCTYPE html>\n\n"
    "<html lang=\"ja\">\n\n"
  );
  sendHeader();
  server->sendContent(
    "<body style=\"padding-top:70px\">\n\n"
  );
  sendNavBar(pageTSL2561);
  
  server->sendContent(
    "<div class=\"container\">\n"
    "  <div class=\"row\"><div class=\"col-sm-10 col-md-8\">\n"
    "    <div class=\"panel panel-primary\">\n"
    "      <div class=\"panel-heading\">\n"
    "        <h3 class=\"panel-title\">TSL2561照度センサー</h3>\n"
    "      </div>\n"
    "      <div class=\"panel-body\">\n"
  );
  if(tsl2561.meas()){
    server->sendContent("        <h4>照度 : " + tsl2561.getLuxStr() + "lux</h4>\n");
  }else{
    server->sendContent(
      "        <h4>TSL2561センサーが接続されていません</h4>\n"
    );
  }
  if(luxLogStatus){
    server->sendContent("        <h4>照度ログ : 動作中\n");
  }else{
    server->sendContent("        <h4>照度ログ : 停止\n");
  }
  server->sendContent(
    "      </div>\n"
    "    </div>\n"
  );
  server->sendContent(
    "    <form method=\"POST\" class=\"form-horizontal\">\n"
  );
  server->sendContent(
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"start\" class=\"btn btn-default btn-block\">開始</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"submit\" name=\"action\" value=\"stop\" class=\"btn btn-default btn-block\">停止</button>\n"
    "        </div>\n"
    "      </div>\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" data-toggle=\"modal\" data-target=\"#clearDialog\">クリア</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" onclick=\"location.href='" + String(luxLogFileName) +
    "'\">ダウンロード</button>\n"
    "        </div>\n"
    "      </div>\n"
    "    </form>\n"
  );
  server->sendContent(  
    "  </div></div><!-- row, col -->\n"
    "</div>\n\n"
  );

  server->sendContent(
    "<div class=\"modal\" id=\"clearDialog\">\n"
    "  <div class=\"modal-dialog\">\n"
    "    <div class=\"modal-content\">\n"
    "      <div class=\"modal-header\">\n"
    "        <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\">&times;</button>\n"
    "        <h4 class=\"modal-title\">ログをクリアしますか？</h4>\n"
    "      </div>\n"
    "      <!-- <div class=\"modal-body\">\n"
    "      </div> -->\n"
    "      <div class=\"modal-footer\">\n"
    "        <form method=\"POST\">\n"
    "          <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\">キャンセル</button>\n"
    "          <button type=\"submit\" class=\"btn btn-primary\" name=\"action\" value=\"clear\">OK</button>\n"
    "        </form>\n"
    "      </div>\n"
    "    </div>\n"
    "  </div>\n"
    "</div>\n\n"
  );

  sendScript();

  server->sendContent(
    "</body>\n"
    "</html>\n"
  );
}


// システムページを送信
void WebPageHandler::handleSystemPage(){
  String actionStr = server->arg("action");
  
  if(actionStr == "formatStorage"){
    #ifdef STORAGE_SPIFFS
    SPIFFS.format();
    #else
    SdFat*SD = new SdFat;
    if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
      SD->wipe();
    }
    delete SD;
    #endif
    
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Format Storage\n");
  }
  
  if(actionStr == "clearOpLog"){
    String path = String("/") + Storage::opLogFileName;
    
    #ifdef STORAGE_SPIFFS
    if(SPIFFS.begin()){
      SPIFFS.remove(path);
      SPIFFS.end();
    }
    #else
    SdFat*SD = new SdFat;
    if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
      if(SD->exists(path.c_str())){
        SD->remove(path.c_str());
      }
    }
    delete SD;
    #endif
    Storage::opLog(timelyTaskManager->getTimeStr() + " Action Clear Op Log\n");
  }
  
  
  server->setContentLength(CONTENT_LENGTH_UNKNOWN);
  server->send(200, "text/html", "");
  
  server->sendContent(
    "<!DOCTYPE html>\n\n"
    "<html lang=\"ja\">\n\n"
  );
  sendHeader();
  server->sendContent(
    "<body style=\"padding-top:70px\">\n\n"
  );
  sendNavBar(pageSystem);
  
  server->sendContent(
    "<div class=\"container\">\n"
    "  <div class=\"row\"><div class=\"col-sm-10 col-md-8\">\n"
    "    <div class=\"panel panel-primary\">\n"
    "      <div class=\"panel-heading\">\n"
    "        <h3 class=\"panel-title\">システム情報</h3>\n"
    "      </div>\n"
    "      <div class=\"panel-body\">\n"
  );
  #ifdef STORAGE_SPIFFS
  SPIFFS.begin();
  struct FSInfo fsInfo;
  SPIFFS.info(fsInfo);
  SPIFFS.end();
  server->sendContent("        <h4>ストレージ : SPIFFS</h4>\n");
  server->sendContent("        <h5>Total : " + String(fsInfo.totalBytes) + " Bytes</h5>\n");
  server->sendContent("        <h5>Used : " + String(fsInfo.usedBytes) + " Bytes</h5>\n");
  #else
  server->sendContent("        <h4>ストレージ : SD</h4>\n");
  //SdFat*SD = new SdFat;
  //if(SD.begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
  //  uint32_t volFree = SD.vol()->freeClusterCount();
  //  uint32_t fs = (volFree*SD.vol()->blocksPerCluster())>>11;
  //  server->sendContent(String() + "        <h5>Free MB : " + String(fs) + "</h4>\n");
  //}
  //delete SD;
  #endif
  server->sendContent("        <h4>Flash動作周波数 : " + String(ESP.getFlashChipSpeed()/1000000) + " MHz</h4>\n");
  server->sendContent("        <h4>Free Heap : " + String(ESP.getFreeHeap()) + " Bytes</h4>\n");
  if(Storage::opLogSerial){
    server->sendContent("        <h4>動作ログ(Serial) : 有効</h4>\n");
  }else{
    server->sendContent("        <h4>動作ログ(Serial) : 無効</h4>\n");
  }
  if(Storage::opLogStorage){
    server->sendContent("        <h4>動作ログ(ストレージ) : 有効</h4>\n");
  }else{
    server->sendContent("        <h4>動作ログ(ストレージ) : 無効</h4>\n");
  }
  
  server->sendContent(
    "      </div>\n"
    "    </div>\n"
    "    <form methos=\"POST\" class=\"form-horizontal\"><fieldset>\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" data-toggle=\"modal\" data-target=\"#formatStorageDialog\">ストレージ消去</button>\n"
    "        </div>\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" data-toggle=\"modal\" data-target=\"#clearOpLogDialog\">動作ログ消去</button>\n"
    "        </div>\n"
    "      </div>\n"
    "      <div class=\"form-group\">\n"
    "        <div class=\"col-xs-6\">\n"
    "          <button type=\"button\" class=\"btn btn-default btn-block\" onclick=\"location.href='"
  );
  server->sendContent(Storage::opLogFileName);
  server->sendContent(
    "'\">動作ログDL</button>\n"
    "        </div>\n"
    "      </div>\n"
    "    </fieldset></form>\n"
    "  </div></div><!-- row, col -->\n"
    "</div>\n\n"
    "<div class=\"modal\" id=\"clearOpLogDialog\">\n"
    "  <div class=\"modal-dialog\">\n"
    "    <div class=\"modal-content\">\n"
    "      <div class=\"modal-header\">\n"
    "        <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\">&times;</button>\n"
    "        <h4 class=\"modal-title\">動作ログを消去しますか？</h4>\n"
    "      </div>\n"
    "      <!-- <div class=\"modal-body\">\n"
    "      </div> -->\n"
    "      <div class=\"modal-footer\">\n"
    "        <form methos=\"POST\">\n"
    "          <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\">キャンセル</button>\n"
    "          <button type=\"submit\" class=\"btn btn-primary\" name=\"action\" value=\"clearOpLog\">OK</button>\n"
    "        </form>\n"
    "      </div>\n"
    "    </div>\n"
    "  </div>\n"
    "</div>\n\n"
    "<div class=\"modal\" id=\"formatStorageDialog\">\n"
    "  <div class=\"modal-dialog\">\n"
    "    <div class=\"modal-content\">\n"
    "      <div class=\"modal-header\">\n"
    "        <button type=\"button\" class=\"close\" data-dismiss=\"modal\" aria-hidden=\"true\">&times;</button>\n"
    "        <h4 class=\"modal-title\">ストレージを消去しますか？</h4>\n"
    "      </div>\n"
    "      <!-- <div class=\"modal-body\">\n"
    "      </div> -->\n"
    "      <div class=\"modal-footer\">\n"
    "        <form methos=\"POST\">\n"
    "          <button type=\"button\" class=\"btn btn-default\" data-dismiss=\"modal\">キャンセル</button>\n"
    "          <button type=\"submit\" class=\"btn btn-primary\" name=\"action\" value=\"formatStorage\">OK</button>\n"
    "        </form>\n"
    "      </div>\n"
    "    </div>\n"
    "  </div>\n"
    "</div>\n\n"
  );

  sendScript();

  server->sendContent(
    "</body>\n"
    "</html>\n"
  );
}




WebPageHandler::WebPageHandler(ESP8266WebServer*s, ArduCamManager*cam, TimelyTaskManager*t, std::function<void(uint8_t)> f){
  server = s;
  arduCamManager = cam;
  timelyTaskManager = t;
  callBack = f;
  motionSensorStatus = false;
  luxLogStatus = false;
}


// クライアントからの要求を処理
void WebPageHandler::handleFile(){
  String path = server->uri();
  String contentType = getContentType(path);

  // いずれかのページに一致する場合はそのページを送信
  if(path == String() + "/" + pageCamera){
    handleCameraPage();
    return;
  }
  
  if(path == String() + "/" + pageCameraSetting){
    handleCameraSettingPage();
    return;
  }

  if(path == String() + "/" + pageTSL2561){
    handleTSL2561Page();
    return;
  }
  
  if(path == String() + "/" + pageMotionSensor){
    handleMotionSensorPage();
    return;
  }

  if(path == String() + "/" + pageSystem){
    handleSystemPage();
    return;
  }

  //ページがない場合、ファイルがあれば返す

  // SPIFFS files
  #ifdef STORAGE_SPIFFS
  if(SPIFFS.begin()){
    if(SPIFFS.exists(path)){
      File file = SPIFFS.open(path, "r");
      server->streamFile(file, contentType);
      file.close();
      return;
    }
    SPIFFS.end();
  }
  // SD card files
  #else
  SdFat*SD = new SdFat;
  if(SD->begin(pinSDCS, SD_SCK_MHZ(spiClkSD))){
    if(SD->exists((char*)path.c_str())){
      File file = SD->open(path);
      if(file){
        server->streamFile(file, contentType);
        file.close();
      }
    }
  }
  delete SD;
  #endif

  // Not found
  server->send(404, "text/plain", "File Not Found");
}




