//---------------------------------
// 2018/6/3

#ifndef WEB_PAGE_HANDLER_H
#define WEB_PAGE_HANDLER_H

#include "Arduino.h"
#include <ESP8266WebServer.h>
#include "Storage.h"
#include "TimelyTaskManager.h"
#include "ArduCamManager.h"


class WebPageHandler{
private:
  static const uint8_t pinSDCS = 2;        // SD SPI CS Pin
  static const uint8_t spiClkSD = 40;      // SD SPI clock MHz
  static const uint8_t pinArduCamCS = 15;  // ArduCAM CS Pin
  static const uint8_t pinSDA = 4;
  static const uint8_t pinSCL = 5;

  static const uint8_t maxFileName = 64;
  

  ESP8266WebServer*server;
  TimelyTaskManager*timelyTaskManager;
  ArduCamManager*arduCamManager;
  std::function<void(uint8_t)> callBack;

  String getContentType(String filename);
  void sendHeader();
  void sendHeaderRefresh(uint8_t t, String link);
  void sendNavBar(const char*page);
  void sendScript();
  void handleCameraPage();
  void handleCameraSettingPage();
  void handleTSL2561Page();
  void handleMotionSensorPage();
  void handleSystemPage();


  
public:
  static const char luxLogFileName[];
  static const char pageCamera[];
  static const char pageCameraSetting[];
  static const char pageTSL2561[];
  static const char pageMotionSensor[];
  static const char pageSystem[];

  // Call back action
  static const uint8_t luxLogStart = 0;
  static const uint8_t luxLogStop = 1;


  bool luxLogStatus;
  bool motionSensorStatus;

  
  WebPageHandler(ESP8266WebServer*s, ArduCamManager*cam, TimelyTaskManager*t, std::function<void(uint8_t)> callBack);
  void handleFile();
  
};

#endif


